/*
MediaInfo, Copyright (C) 1999-2000 Jean-Baptiste M. Queru

This program is free software; you can redistribute it and/or 
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License 
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Contact the author at:
Jean-Baptiste M. Queru, 31 Camino Alto, Millbrae, CA 94030, USA
or by e-mail at : djaybee@cyberdude.com
*/

#include "MediaInfoApp.h"
#include "MediaInfoView.h"
#include "MediaInfoDefs.h"

#include <ScrollView.h>
#include <MenuBar.h>
#include <MenuItem.h>
#include <Menu.h>

MediaInfoWindow::MediaInfoWindow(BRect f):BWindow(f,BINARY_NAME,B_DOCUMENT_WINDOW_LOOK,B_NORMAL_WINDOW_FEEL,0) {
	BMenuBar*mb;
	BMenu*m;
	BMenuItem*mi;
	mb=new BMenuBar(BRect(),"menubar");
	m=new BMenu("File");
	mi=new BMenuItem("About...",new BMessage(B_ABOUT_REQUESTED));
	mi->SetTarget(be_app);
	m->AddItem(mi);
	mi=new BMenuItem("New Window",new BMessage(NEW_WINDOW),'N');
	mi->SetTarget(be_app);
	m->AddItem(mi);
	mi=new BMenuItem("Close",new BMessage(B_QUIT_REQUESTED),'W');
	m->AddItem(mi);
	mi=new BMenuItem("Quit",new BMessage(B_QUIT_REQUESTED),'Q');
	mi->SetTarget(be_app);
	m->AddItem(mi);
	mb->AddItem(m);
	AddChild(mb);
	BRect r(Bounds());
	r.top+=mb->Frame().Height()+1;
	r.right-=B_V_SCROLL_BAR_WIDTH;
	AddChild(new BScrollView("",new MediaInfoView(r),B_FOLLOW_ALL_SIDES,0,false,true,B_NO_BORDER));
}

bool MediaInfoWindow::QuitRequested() {
	be_app->PostMessage(WINDOW_GONE);
	return true;
}

MediaInfoApp::MediaInfoApp():BApplication("application/x-vnd.djaybee.MediaInfo") {
	numwindows=0;
	about_box=NULL;
}

void MediaInfoApp::ReadyToRun() {
	numwindows++;
	(new MediaInfoWindow(BRect(100,100,399,399)))->Show();
}

void MediaInfoApp::AboutRequested() {
	if (about_box==NULL) {
		BRect f(200,200,200,200);
		for (int i=0;i<CountWindows();i++) {
			f=WindowAt(i)->Frame(); // race conditions here? how to fix?
			if (WindowAt(i)->IsActive()) {
				break;
			}
		}
		f.OffsetBy(20,20);
		(about_box=new MediaInfoAboutBox(f))->Show();
	} else {
		about_box->Activate(); // race condition?
	}
}

void MediaInfoApp::MessageReceived(BMessage*m) {
	switch (m->what) {
		case NEW_WINDOW : {
			numwindows++;
			BRect f(100,100,399,399);
			for (int i=0;i<CountWindows();i++) {
				f=WindowAt(i)->Frame(); // race conditions here? how to fix?
				if (WindowAt(i)->IsActive()) {
					break;
				}
			}
			f.OffsetBy(20,20);
			(new MediaInfoWindow(f))->Show();
			break;
		}
		case WINDOW_GONE : {
			if (--numwindows==0) {
				PostMessage(B_QUIT_REQUESTED); // bug here
			}
			break;
		}
		case ABOUT_BOX_GONE : {
			about_box=NULL;
			break;
		}
		default : {
			BApplication::MessageReceived(m);
			break;
		}
	}
}

int main() {
	MediaInfoApp().Run();
}
