/*
MediaInfo, Copyright (C) 1999-2000 Jean-Baptiste M. Queru

This program is free software; you can redistribute it and/or 
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License 
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Contact the author at:
Jean-Baptiste M. Queru, 31 Camino Alto, Millbrae, CA 94030, USA
or by e-mail at : djaybee@cyberdude.com
*/

#include "MediaInfoView.h"

#include <Application.h>
#include <Window.h>
#include <TextView.h>
#include <Entry.h>
#include <MediaFile.h>
#include <MediaTrack.h>
#include <ScrollView.h>
#include <stdio.h>
#include <string.h>

MediaInfoView::MediaInfoView(BRect r):BTextView(r,"",r,B_FOLLOW_ALL_SIDES,B_WILL_DRAW|B_FRAME_EVENTS) {
	BRect f=Bounds();
	f.InsetBy(3,3);
	SetTextRect(f);
	SetText("Drop a file here");
	MakeEditable(false);
	MakeSelectable(true);
	SetWordWrap(true);
}

void MediaInfoView::AttachedToWindow() {
	MakeFocus();
}

void MediaInfoView::FrameResized(float x,float y) {
	BRect f=Bounds();
	f.InsetBy(3,0);
	f.top=3;
	SetTextRect(f);
	BTextView::FrameResized(x,y);
}

void MediaInfoView::MessageReceived(BMessage*m) {
	prefix[0]='\0';
	if (m->WasDropped()) {
		Delete(0,TextLength());
		entry_ref ref;
		if (m->FindRef("refs",&ref)==B_OK) {
			Select(0,0);
			Window()->SetTitle(ref.name);
			BMediaFile*mf=new BMediaFile(&ref);
			if (mf->InitCheck()==B_OK) {
				PrintMediaFile(mf);
			} else {
				PrintLine("Not a valid media file\n");
			}
			delete mf;
			return;
		}
	}
	BTextView::MessageReceived(m);
}

// high-level

void MediaInfoView::PrintMediaFile(BMediaFile*mf) {
	status_t ret;
	PrintLine("BMediaFile info :\n");
	const char* copyright=mf->Copyright();
	if (copyright==NULL) {
		PrintLine("No copyright info\n");
	} else {
		PrintLine("copyright : %s\n",copyright);
	}
	int32 ntracks=mf->CountTracks();
	PrintLine("%ld track(s)\n",ntracks);
	media_file_format mff;
	ret=mf->GetFileFormatInfo(&mff);
	if (ret!=B_OK) {
		PrintLine("Error in GetFormatFileInfo : $s\n",strerror(ret));
	} else {
		PrintMediaFileFormat(mff);
	}
	for (int32 i=0;i<ntracks;i++) {
		PrintLine("\nBMediaTrack #%ld info :\n",i);
		BMediaTrack* mt=mf->TrackAt(i);
		if (mt==NULL) {
			PrintLine("NULL track\n");
			continue;
		}
		ret=mt->InitCheck();
		if (ret!=B_OK) {
			PrintLine("error initializing track : %s\n",strerror(ret));
			continue;
		}
		PrintMediaTrack(mt);
	}
	ret=mf->ReleaseAllTracks();
	if (ret!=B_OK) {
		PrintLine("\nerror in ReleaseAllTracks : %s\n",strerror(ret));
	}
}

void MediaInfoView::PrintMediaTrack(BMediaTrack*mt) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	status_t ret;
	PrintLine("BMediaTrack length : %Ld frames\n",mt->CountFrames());
	PrintLine("BMediaTrack duration : %Ld microseconds\n",mt->Duration());
	media_format mf;
	ret=mt->EncodedFormat(&mf);
	if (ret!=B_OK) {
		PrintLine("error getting encoded media format : %s\n",strerror(ret));
	} else {
		strcat(prefix,"encoded ");
		PrintMediaFormat(mf);
	}
	memcpy(prefix,tmppfx,BUFFER_SIZE);
	ret=mt->DecodedFormat(&mf);
	if (ret!=B_OK) {
		PrintLine("error getting decoded media format : %s\n",strerror(ret));
	} else {
		strcat(prefix,"decoded ");
		PrintMediaFormat(mf);
	}
	memcpy(prefix,tmppfx,BUFFER_SIZE);
	media_codec_info mci;
	ret=mt->GetCodecInfo(&mci);
	if (ret!=B_OK) {
		PrintLine("error getting media : %s\n",strerror(ret));
	} else {
		PrintMediaCodecInfo(mci);
	}
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaFormat(const media_format& mf) {
	char tmppfx[BUFFER_SIZE];
	char tmppfx2[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"media_format ");
	PrintMediaFormatType(mf.type);
	memcpy(tmppfx2,prefix,BUFFER_SIZE);
	strcat(prefix,"require ");
	PrintMediaFormatFlags(mf.require_flags);
	memcpy(prefix,tmppfx2,BUFFER_SIZE);
	strcat(prefix,"deny ");
	PrintMediaFormatFlags(mf.deny_flags);
	memcpy(prefix,tmppfx2,BUFFER_SIZE);
	switch(mf.type) {
		case B_MEDIA_RAW_AUDIO : PrintMediaMultiAudioFormat(mf.u.raw_audio);break;
		case B_MEDIA_ENCODED_AUDIO : PrintMediaEncodedAudioFormat(mf.u.encoded_audio);break;
		case B_MEDIA_RAW_VIDEO : PrintMediaRawVideoFormat(mf.u.raw_video);break;
		case B_MEDIA_ENCODED_VIDEO : PrintMediaEncodedVideoFormat(mf.u.encoded_video);break;
		default : PrintLine("unknown media format type\n");
	}
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaFileFormat(const media_file_format& mff) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"media_file_format ");
	PrintMediaFileFormatCapabilities(mff.capabilities);
	PrintMediaFormatFamily(mff.family);
	PrintLine("version : %ld\n",mff.version);
	PrintLine("mime type : %s\n",mff.mime_type);
	PrintLine("pretty name : %s\n",mff.pretty_name);
	PrintLine("short name : %s\n",mff.short_name);
	PrintLine("file extension : %s\n",mff.file_extension);
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaRawAudioFormat(const media_raw_audio_format&mraf) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"media_raw_audio_format ");
	PrintLine("frame rate : %f\n",mraf.frame_rate);
	PrintLine("channel count : %lu\n",mraf.channel_count);
	PrintMediaRawAudioFormatFormat(mraf.format);
	PrintByteOrder(mraf.byte_order);
	PrintLine("buffer size : %lu\n",mraf.buffer_size);
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaMultiAudioInfo(const media_multi_audio_info&mmai) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"media_multi_audio_info ");
	PrintLine("channel mask : %lx\n",mmai.channel_mask);
	PrintLine("valid bits : %hd\n",mmai.valid_bits);
	PrintLine("matrix mask : %hx\n",mmai.matrix_mask);
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaMultiAudioFormat(const media_multi_audio_format& mmaf) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"media_multi_audio_format ");
	PrintMediaRawAudioFormat(mmaf);
	PrintMediaMultiAudioInfo(mmaf);
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaEncodedAudioFormat(const media_encoded_audio_format&meaf) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"media_encoded_audio_format ");
	PrintMediaRawAudioFormat(meaf.output);
	PrintLine("bit rate : %f\n",meaf.bit_rate);
	PrintLine("frame size : %lu\n",meaf.frame_size);
	PrintMediaMultiAudioInfo(meaf.multi_info);
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaRawVideoFormat(const media_raw_video_format&mrvf) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"media_raw_video_format ");
	PrintLine("field rate : %f\n",mrvf.field_rate);
	PrintLine("interlace : %lu\n",mrvf.interlace);
	PrintLine("first active : %lu\n",mrvf.first_active);
	PrintLine("last active : %lu\n",mrvf.last_active);
	PrintMediaRawVideoFormatOrientation(mrvf.orientation);
	PrintLine("pixel width_aspect : %hu\n",mrvf.pixel_width_aspect);
	PrintLine("pixel height_aspect : %hu\n",mrvf.pixel_height_aspect);
	PrintMediaVideoDisplayInfo(mrvf.display);
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaVideoDisplayInfo(const media_video_display_info&mvdi) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"media_video_display_info ");
	PrintColorSpace(mvdi.format);
	PrintLine("line width : %lu\n",mvdi.line_width);
	PrintLine("line count : %lu\n",mvdi.line_count);
	PrintLine("bytes per row : %lu\n",mvdi.bytes_per_row);
	PrintLine("pixel offset : %lu\n",mvdi.pixel_offset);
	PrintLine("line offset : %lu\n",mvdi.line_offset);
	PrintMediaVideoDisplayInfoFlags(mvdi.flags);
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaEncodedVideoFormat(const media_encoded_video_format&mevf) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"media_encoded_video_format ");
	PrintMediaRawVideoFormat(mevf.output);
	PrintLine("avg bit rate : %f\n",mevf.avg_bit_rate);
	PrintLine("max bit rate : %f\n",mevf.max_bit_rate);
	PrintLine("frame size : %lu\n",mevf.frame_size);
	PrintLine("forward history : %hd\n",mevf.forward_history);
	PrintLine("backward history : %hd\n",mevf.backward_history);
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaCodecInfo(const media_codec_info&mci) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"media_codec_info ");
	PrintLine("pretty name : %s\n",mci.pretty_name);
	PrintLine("short name : %s\n",mci.short_name);
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

// mid-level

void MediaInfoView::PrintMediaRawVideoFormatOrientation(uint32 o) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"orientation : ");
	switch(o) {
		case B_VIDEO_TOP_LEFT_RIGHT : PrintLine("top left-right\n");break;
		case B_VIDEO_BOTTOM_LEFT_RIGHT : PrintLine("bottom left-right\n");break;
		default : PrintLine("undefined constant\n");break;
	}
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaVideoDisplayInfoFlags(uint32 f) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"flags : ");
	PrintLine("[begin]\n");
	if (f&B_F1_DOMINANT) PrintLine("field #1 dominant\n");
	if (f&B_F2_DOMINANT) PrintLine("field #2 dominant\n");
	if (f&B_TOP_SCANLINE_F1) PrintLine("top scanline in field #1\n");
	if (f&B_TOP_SCANLINE_F2) PrintLine("top scanline in field #2\n");
	PrintLine("[end]\n");
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaRawAudioFormatFormat(uint32 f) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"format : ");
	switch(f) {
		case media_raw_audio_format::B_AUDIO_FLOAT : PrintLine("float\n");break;
		case media_raw_audio_format::B_AUDIO_INT : PrintLine("int\n");break;
		case media_raw_audio_format::B_AUDIO_SHORT : PrintLine("short\n");break;
		case media_raw_audio_format::B_AUDIO_UCHAR : PrintLine("uchar\n");break;
		case media_raw_audio_format::B_AUDIO_CHAR : PrintLine("char\n");break;
		default : PrintLine("undefined constant\n");break;
	}
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintByteOrder(uint32 bo) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"byte_order : ");
	switch(bo) {
		case B_MEDIA_BIG_ENDIAN : PrintLine("big-endian\n");break;
		case B_MEDIA_LITTLE_ENDIAN : PrintLine("little-endian\n");break;
		default : PrintLine("undefined constant\n");break;
	}
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaFormatFlags(uint16 flags) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"media_format_flags : ");
	PrintLine("[begin]\n");
	if (flags&B_MEDIA_RETAINED_DATA) PrintLine("retained data\n");
	if (flags&B_MEDIA_MULTIPLE_BUFFERS) PrintLine("multiple buffers\n");
	if (flags&B_MEDIA_CONTIGUOUS_BUFFER) PrintLine("contiguous buffers\n");
	if (flags&B_MEDIA_LINEAR_UPDATES) PrintLine("linear updates\n");
	PrintLine("[end]\n");
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaFormatType(media_type mt) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"type : ");
	switch(mt) {
		case B_MEDIA_NO_TYPE : PrintLine("no media type\n");break;
		case B_MEDIA_UNKNOWN_TYPE : PrintLine("unknown type\n");break;
		case B_MEDIA_RAW_AUDIO : PrintLine("raw audio\n");break;
		case B_MEDIA_RAW_VIDEO : PrintLine("raw video\n");break;
		case B_MEDIA_VBL : PrintLine("VBL\n");break;
		case B_MEDIA_TIMECODE : PrintLine("timecode\n");break;
		case B_MEDIA_MIDI : PrintLine("midi\n");break;
		case B_MEDIA_TEXT : PrintLine("text\n");break;
		case B_MEDIA_HTML : PrintLine("html\n");break;
		case B_MEDIA_MULTISTREAM : PrintLine("multistream\n");break;
		case B_MEDIA_PARAMETERS : PrintLine("parameters\n");break;
		case B_MEDIA_ENCODED_AUDIO : PrintLine("encoded audio\n");break;
		case B_MEDIA_ENCODED_VIDEO : PrintLine("encoded video\n");break;
		default : PrintLine("undefined constant\n");break;
	}
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaFormatFamily(media_format_family mff) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"family : ");
	switch(mff) {
		case B_ANY_FORMAT_FAMILY : PrintLine("any\n");break;
		case B_BEOS_FORMAT_FAMILY : PrintLine("BeOS\n");break;
		case B_QUICKTIME_FORMAT_FAMILY : PrintLine("quicktime\n");break;
		case B_AVI_FORMAT_FAMILY : PrintLine("avi\n");break;
		case B_ASF_FORMAT_FAMILY : PrintLine("asf\n");break;
		case B_MPEG_FORMAT_FAMILY : PrintLine("mpeg\n");break;
		case B_WAV_FORMAT_FAMILY : PrintLine("wav\n");break;
		case B_AIFF_FORMAT_FAMILY : PrintLine("aiff\n");break;
		case B_MISC_FORMAT_FAMILY : PrintLine("misc\n");break;
		default : PrintLine("undefined constant\n");break;
	}
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintMediaFileFormatCapabilities(uint32 capabilities) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"capabilities : ");
	PrintLine("[begin]\n");
	if (capabilities&media_file_format::B_READABLE) PrintLine("readable\n");
	if (capabilities&media_file_format::B_WRITABLE) PrintLine("writable\n");
	if (capabilities&media_file_format::B_PERFECTLY_SEEKABLE) PrintLine("perfectly seekable\n");
	if (capabilities&media_file_format::B_IMPERFECTLY_SEEKABLE) PrintLine("imperfectly seekable\n");
	if (capabilities&media_file_format::B_KNOWS_RAW_VIDEO) PrintLine("knows raw video\n");
	if (capabilities&media_file_format::B_KNOWS_RAW_AUDIO) PrintLine("knows raw audio\n");
	if (capabilities&media_file_format::B_KNOWS_MIDI) PrintLine("knows midi\n");
	if (capabilities&media_file_format::B_KNOWS_ENCODED_VIDEO) PrintLine("knows encoded video\n");
	if (capabilities&media_file_format::B_KNOWS_ENCODED_AUDIO) PrintLine("knows encoded audio\n");
	if (capabilities&media_file_format::B_KNOWS_OTHER) PrintLine("knows other\n");
	if (capabilities&media_file_format::B_KNOWS_ANYTHING) PrintLine("knows anything\n");
	PrintLine("[end]\n");
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

void MediaInfoView::PrintColorSpace(color_space cs) {
	char tmppfx[BUFFER_SIZE];
	memcpy(tmppfx,prefix,BUFFER_SIZE);
	strcat(prefix,"color space : ");
	switch(cs) {
		case B_NO_COLOR_SPACE : PrintLine("no color space\n");break;
		case B_RGB32 : PrintLine("RGB32\n");break;
		case B_RGBA32 : PrintLine("RGBA32\n");break;
		case B_RGB24 : PrintLine("RGB24\n");break;
		case B_RGB16 : PrintLine("RGB16\n");break;
		case B_RGB15 : PrintLine("RGB15\n");break;
		case B_RGBA15 : PrintLine("RGBA15\n");break;
		case B_CMAP8 : PrintLine("CMAP8\n");break;
		case B_GRAY8 : PrintLine("GRAY8\n");break;
		case B_GRAY1 : PrintLine("GRAY1\n");break;
		case B_RGB32_BIG : PrintLine("RGB32 (big)\n");break;
		case B_RGBA32_BIG : PrintLine("RGBA32 (big)\n");break;
		case B_RGB24_BIG : PrintLine("RGB24 (big)\n");break;
		case B_RGB16_BIG : PrintLine("RGB16 (big)\n");break;
		case B_RGB15_BIG : PrintLine("RGB15 (big)\n");break;
		case B_RGBA15_BIG : PrintLine("RGBA15 (big)\n");break;
		case B_YCbCr422 : PrintLine("YCbCr422\n");break;
		case B_YCbCr411 : PrintLine("YCbCr411\n");break;
		case B_YCbCr444 : PrintLine("YCbCr444\n");break;
		case B_YCbCr420 : PrintLine("YCbCr420\n");break;
		case B_YUV422 : PrintLine("YUV422\n");break;
		case B_YUV411 : PrintLine("YUV411\n");break;
		case B_YUV444 : PrintLine("YUV444\n");break;
		case B_YUV420 : PrintLine("YUV420\n");break;
		case B_YUV9 : PrintLine("YUV9\n");break;
		case B_YUV12 : PrintLine("YUV12\n");break;
		case B_UVL24 : PrintLine("UVL24\n");break;
		case B_UVL32 : PrintLine("UVL32\n");break;
		case B_UVLA32 : PrintLine("UVLA32\n");break;
		case B_LAB24 : PrintLine("LAB24\n");break;
		case B_LAB32 : PrintLine("LAB32\n");break;
		case B_LABA32 : PrintLine("LABA32\n");break;
		case B_HSI24 : PrintLine("HSI24\n");break;
		case B_HSI32 : PrintLine("HSI32\n");break;
		case B_HSIA32 : PrintLine("HSIA32\n");break;
		case B_HSV24 : PrintLine("HSV24\n");break;
		case B_HSV32 : PrintLine("HSV32\n");break;
		case B_HSVA32 : PrintLine("HSVA32\n");break;
		case B_HLS24 : PrintLine("HLS24\n");break;
		case B_HLS32 : PrintLine("HLS32\n");break;
		case B_HLSA32 : PrintLine("HLSA32\n");break;
		case B_CMY24 : PrintLine("CMY24\n");break;
		case B_CMY32 : PrintLine("CMY32\n");break;
		case B_CMYA32 : PrintLine("CMYA32\n");break;
		case B_CMYK32 : PrintLine("CMYK32\n");break;
		default : PrintLine("undefined constant\n");break;
	}
	memcpy(prefix,tmppfx,BUFFER_SIZE);
}

// low-level

void MediaInfoView::PrintLine(const char*fmt) {
	snprintf(buffer,BUFFER_SIZE,fmt);
	Insert(prefix);
	Insert(buffer);
}

void MediaInfoView::PrintLine(const char*fmt,const char*s) {
	snprintf(buffer,BUFFER_SIZE,fmt,s);
	Insert(prefix);
	Insert(buffer);
}

void MediaInfoView::PrintLine(const char*fmt,int16 s) {
	snprintf(buffer,BUFFER_SIZE,fmt,s);
	Insert(prefix);
	Insert(buffer);
}

void MediaInfoView::PrintLine(const char*fmt,uint16 us) {
	snprintf(buffer,BUFFER_SIZE,fmt,us);
	Insert(prefix);
	Insert(buffer);
}

void MediaInfoView::PrintLine(const char*fmt,int32 l) {
	snprintf(buffer,BUFFER_SIZE,fmt,l);
	Insert(prefix);
	Insert(buffer);
}

void MediaInfoView::PrintLine(const char*fmt,uint32 ul) {
	snprintf(buffer,BUFFER_SIZE,fmt,ul);
	Insert(prefix);
	Insert(buffer);
}

void MediaInfoView::PrintLine(const char*fmt,int64 ll) {
	snprintf(buffer,BUFFER_SIZE,fmt,ll);
	Insert(prefix);
	Insert(buffer);
}

void MediaInfoView::PrintLine(const char*fmt,float f) {
	snprintf(buffer,BUFFER_SIZE,fmt,f);
	Insert(prefix);
	Insert(buffer);
}
