/*
MediaInfo, Copyright (C) 1999-2000 Jean-Baptiste M. Queru

This program is free software; you can redistribute it and/or 
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License 
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Contact the author at:
Jean-Baptiste M. Queru, 31 Camino Alto, Millbrae, CA 94030, USA
or by e-mail at : djaybee@cyberdude.com
*/

#ifndef MEDIAINFOVIEW_H_
#define MEDIAINFOVIEW_H_

#include <TextView.h>
#include <MediaDefs.h>
#include <MediaFile.h>
#include <MediaTrack.h>
#include <MediaFormats.h>

#define BUFFER_SIZE 512

class MediaInfoView : public BTextView {
public:
	MediaInfoView(BRect r);
private:
	void AttachedToWindow();
	void MessageReceived(BMessage*);
	void FrameResized(float,float);

// high-level functions
	void PrintMediaFile(BMediaFile*);
	void PrintMediaTrack(BMediaTrack*);
	void PrintMediaFileFormat(const media_file_format&);
	void PrintMediaFormat(const media_format&);
	void PrintMediaMultiAudioFormat(const media_multi_audio_format&);
	void PrintMediaMultiAudioInfo(const media_multi_audio_info&);
	void PrintMediaRawAudioFormat(const media_raw_audio_format&);
	void PrintMediaEncodedAudioFormat(const media_encoded_audio_format&);
	void PrintMediaRawVideoFormat(const media_raw_video_format&);
	void PrintMediaVideoDisplayInfo(const media_video_display_info&);
	void PrintMediaEncodedVideoFormat(const media_encoded_video_format&);
	void PrintMediaCodecInfo(const media_codec_info&);

// mid-level functions
	void PrintColorSpace(color_space);
	void PrintMediaFileFormatCapabilities(uint32);
	void PrintMediaFormatFamily(media_format_family);
	void PrintMediaFormatType(media_type);
	void PrintMediaFormatFlags(uint16);
	void PrintByteOrder(uint32);
	void PrintMediaRawAudioFormatFormat(uint32);
	void PrintMediaRawVideoFormatOrientation(uint32);
	void PrintMediaVideoDisplayInfoFlags(uint32);

// low-level functions
	void PrintLine(const char*);
	void PrintLine(const char*,const char*);
	void PrintLine(const char*,int16);
	void PrintLine(const char*,uint16);
	void PrintLine(const char*,int32);
	void PrintLine(const char*,uint32);
	void PrintLine(const char*,int64);
	void PrintLine(const char*,float);

	char buffer[BUFFER_SIZE];
	char prefix[BUFFER_SIZE];
};

#endif
